/*                                                                            */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <clib/alib_protos.h>
	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/intuition.h>
	#include <proto/utility.h>
	#include <reaction/reaction.h>
	#include <reaction/reaction_macros.h>

	#include <proto/window.h>
	#include <classes/window.h>

	#include <proto/layout.h>
	#include <gadgets/layout.h>

	#include <proto/button.h>
	#include <gadgets/button.h>

	#include <proto/slider.h>
	#include <gadgets/slider.h>

	#include <string.h>
	#include <stdio.h>

/*----------------------------------------------------------------------------*/
/* Konstanten und Makros                                                      */
/*----------------------------------------------------------------------------*/

	#define GID_END  3


#ifdef __amigaos4__
	#define BASEDEF(base) struct Library *base##Base = NULL; struct base##IFace *I##base = NULL
	#define OPENLIB(base,name) if (!(base##Base = OpenLibrary (name,0)) || !(I##base = (struct base##IFace *)GetInterface ((struct Library *)base##Base,"main",1,NULL))) ok = FALSE
	#define CLOSELIB(base) if (I##base) DropInterface ((struct Interface *)I##base); if (base##Base) CloseLibrary (base##Base)
#else
	#define BASEDEF(base) struct Library *base##Base = NULL
	#define OPENLIB(base,name) if (!(base##Base = OpenLibrary (name,0))) ok = FALSE
	#define CLOSELIB(base) if (base##Base) CloseLibrary (base##Base)
#endif

/*----------------------------------------------------------------------------*/
/* Type definitions                                                           */
/*----------------------------------------------------------------------------*/

	struct gadwin {
		struct Gadget *gad;
		struct Window *win;
		};

/*----------------------------------------------------------------------------*/
/* Globale Variablen                                                          */
/*----------------------------------------------------------------------------*/

	BASEDEF(Window);
	BASEDEF(Layout);
	BASEDEF(Button);
	BASEDEF(Slider);

	const UWORD putchproc[] = {0x16c0,0x4e75};	/* move.b d0,(a3)+ ; rts */

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL open_libs (void)

	{
	BOOL ok = TRUE;

	OPENLIB(Window,"window.class");
	OPENLIB(Layout,"gadgets/layout.gadget");
	OPENLIB(Button,"gadgets/button.gadget");
	OPENLIB(Slider,"gadgets/slider.gadget");

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void close_libs (void)

	{
	CLOSELIB(Window);
	CLOSELIB(Layout);
	CLOSELIB(Button);
	CLOSELIB(Slider);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	__saveds void dispfunct (struct Hook *hook,struct Gadget *obj,struct TagItem *msg)

	{
	struct gadwin *data;
	long val;
	short args[3];

	data = hook->h_Data;
	val = GetTagData (SLIDER_Level,0,msg);
	val -= 100;
	if (val < 0)
		{
		val = -val;
		args[0] = '-';
		}
	else
		args[0] = ' ';
	args[1] = val / 100;
	args[2] = val % 100;
	SetGadgetAttrs (data->gad,data->win,NULL,GA_Text,"%c%d.%02d",BUTTON_VarArgs,args,TAG_END);
	}

/*----------------------------------------------------------------------------*/
/* Hauptprogramm                                                              */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	Object *winobj;
	struct Screen *scr;
	struct Window *win;
	ULONG winsig,sigs;
	BOOL cont;
	ULONG result;
	struct Gadget *slidgad;
	struct Gadget *dispgad;
	struct Hook disphook;
	struct gadwin dispdata;

	if (!open_libs())
		{
		Printf ("cannot open required BOOPSI classes\n");
		close_libs();
		return (20);
		}

	if (scr = LockPubScreen (NULL))
		{

		if (winobj = WindowObject,
				WA_CustomScreen,scr,
				WA_Title,"ReAct",
				WA_Width,scr->Width / 4,
				WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_SIZEBBOTTOM | WFLG_SIZEGADGET | WFLG_ACTIVATE,
				WA_IDCMP,IDCMP_VANILLAKEY,
				WINDOW_Position,WPOS_CENTERSCREEN,
				WINDOW_ParentGroup,VGroupObject,
					StartHGroup,
						LAYOUT_SpaceOuter,TRUE,
						LAYOUT_AddChild,slidgad = SliderObject,
							SLIDER_Orientation,SLIDER_HORIZONTAL,
							SLIDER_Min,0,
							SLIDER_Max,200,
							SLIDER_Ticks,9,
							SLIDER_ShortTicks,TRUE,
							SLIDER_KnobDelta,10,
						EndObject,
						LAYOUT_AddChild,dispgad = ButtonObject,
							GA_Text,"-1.00",
							GA_ReadOnly,TRUE,
							BUTTON_BevelStyle,BVS_NONE,
							BUTTON_Transparent,TRUE,
						EndObject,
						CHILD_WeightedWidth,0,
					EndHGroup,
					StartHGroup,
						LAYOUT_SpaceOuter,TRUE,
						LAYOUT_FixedVert,FALSE,
						LAYOUT_EvenSize,TRUE,
						LAYOUT_AddChild,
							Button("Ende",GID_END),
					EndHGroup,
				EndVGroup,
			EndWindow)

			{
			if (win = RA_OpenWindow ((Object *)winobj))
				{

				dispdata.gad        = dispgad;
				dispdata.win        = win;
				disphook.h_Entry    = (HOOKFUNC)HookEntry;
				disphook.h_SubEntry = (HOOKFUNC)dispfunct;
				disphook.h_Data     = &dispdata;
				SetAttrs (slidgad,SLIDER_DispHook,&disphook,TAG_END);
				SetGadgetAttrs (slidgad,win,NULL,SLIDER_Level,100,TAG_END);

				GetAttr (WINDOW_SigMask, winobj, &winsig);

				cont = TRUE;
				do	{
					sigs = Wait (winsig | SIGBREAKF_CTRL_C);

					if (sigs & SIGBREAKF_CTRL_C)
						cont = FALSE;

					if (sigs & winsig)
						while ((result = RA_HandleInput ((Object *)winobj, NULL)) != WMHI_LASTMSG)
							{
							switch (result & WMHI_CLASSMASK)
								{
							case WMHI_GADGETUP:
								switch (result & WMHI_GADGETMASK)
									{
								case GID_END:
									cont = FALSE;
									break;
									}
								break;
							case WMHI_CLOSEWINDOW:
								cont = FALSE;
								break;
							case WMHI_VANILLAKEY:
								if ((result & WMHI_KEYMASK) == 0x1b)
									cont = FALSE;
								break;
							default:
								break;
								}
							}
					}
				while (cont);
				}

			DisposeObject (winobj);
			}

		UnlockPubScreen (NULL,scr);
		}

	close_libs();

	return (0);
	}

/*----------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                       */
/*----------------------------------------------------------------------------*/
